// Bridge.cpp: implementation of the CBridge class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Map Studio MAX R1.h"
#include "Bridge.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL (CBridge, CObject, 1)

CBridge::CBridge (CPoint ptFrom, CPoint ptTo, UINT nWidth, UINT lines_distance, COLORREF crColor)
{
    m_StartingPoint = ptFrom;
    m_EndPoint = ptTo;
    m_LineWidth = nWidth;
    m_LineColor = crColor;
	distance_porpotion = lines_distance;
}

void CBridge::Serialize (CArchive& ar)
{
    CObject::Serialize (ar);

    if (ar.IsStoring ())
        ar << m_StartingPoint << m_EndPoint << m_LineWidth << distance_porpotion << (DWORD) m_LineColor;
    else
        ar >> m_StartingPoint >> m_EndPoint >> m_LineWidth >> distance_porpotion >> (DWORD) m_LineColor;
}


// Get methods
CPoint CBridge::GetStartingPoint()
{
	return m_StartingPoint;
}

CPoint CBridge::GetEndPoint()
{
	return m_EndPoint;
}

UINT CBridge::GetLineWidth()
{
	return m_LineWidth;
}

COLORREF CBridge::GetBridgeColor()
{
	return m_LineColor;
}

int CBridge::GetBridgeWidth()
{
	return distance_porpotion;
}

// Set methods
void CBridge::SetStartingPoint(CPoint start_point)
{
	m_StartingPoint = start_point;
}

void CBridge::SetEndPoint(CPoint end_point)
{
	m_EndPoint = end_point;
}

void CBridge::SetLineWidth(UINT line_width)
{
	m_LineWidth = line_width;
}

void CBridge::SetBridgeColor(COLORREF line_color)
{
	m_LineColor = line_color;
}

void CBridge::SetBridgeWidth(int bridge_width)
{
	distance_porpotion = bridge_width;
}

bool CBridge::HitTest(CPoint point)
{
    // check handles
    CRect R;

    R = CRect(m_StartingPoint, m_EndPoint );
    R.NormalizeRect();
    CRect VicX = R;
    CRect VicY = R;

    VicX.InflateRect( 5, 0 );
    VicY.InflateRect( 0, 5 );

    if (!VicX.PtInRect(point) && !VicY.PtInRect(point))
		return false;

    // check distance from the point to the line by x and by y
    
	int dx = m_EndPoint.x - m_StartingPoint.x;
    int dy = m_EndPoint.y - m_StartingPoint.y;
    double r = sqrt(dx*dx + dy*dy);

    // compute coordinates relative to the origin of the line
    int x1 = point.x - m_StartingPoint.x;
    int y1 = point.y - m_StartingPoint.y;

    // compute coordinates relative to the line
	double x2 = (x1*dx + y1*dy)/r;
    double y2 = (-x1*dy + y1*dx)/r;


	if ((x2>=0) && (x2<=r) && (y2<=0) && (y2 >= -distance_porpotion))
		return true;

    return false;
}