// BridgeView.cpp: implementation of the CBridgeView class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Map Studio MAX R1.h"
#include "BridgeView.h"
#include <math.h>
#include <stdlib.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// define pi
double pi = 3.1415926535;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBridgeView::CBridgeView()
{

}

CBridgeView::CBridgeView(CBridge *bridge)
{
    m_StartingPoint = bridge->GetStartingPoint();
    m_EndPoint = bridge->GetEndPoint();
    m_LineWidth = bridge->GetLineWidth();
    m_LineColor = bridge->GetBridgeColor();
	distance_porpotion = bridge->GetBridgeWidth();
}

CBridgeView::~CBridgeView()
{

}


void CBridgeView::Draw (CDC* pDC)
{
    CPen pen (PS_SOLID, m_LineWidth, m_LineColor);
    CPen* pOldPen = pDC->SelectObject (&pen);
	pDC->SelectObject (&pen);

	// define the horizontal and vertical distance for the line draw
	distance_x = m_EndPoint.x-m_StartingPoint.x;
	distance_y = m_EndPoint.y-m_StartingPoint.y;

	int distance_porpotion_collection[3];
	distance_porpotion_collection[0] = 2;
	distance_porpotion_collection[1] = distance_porpotion - 2;
	distance_porpotion_collection[2] = distance_porpotion;

	// avoid devide by zero
	if (distance_x == 0)
	{	
		for (int n =0; n<3 ;n++)
		{
			x = m_StartingPoint.x + distance_porpotion_collection[n];
			y = m_StartingPoint.y;

			pDC->MoveTo(m_StartingPoint);
			pDC->LineTo(m_EndPoint);

			x1 = x + (int)distance_x;
			y1 = y + (int)distance_y;
			
			pDC->MoveTo(CPoint(x,y));
			pDC->LineTo(CPoint(x1,y1));
		}
		return;
	}

	// to Disable visual artifact
	if (distance_y == 0)
	{	
		for (int n =0; n<3 ;n++)
		{
			x = m_StartingPoint.x ;
			y = m_StartingPoint.y - distance_porpotion_collection[n];

			pDC->MoveTo(m_StartingPoint);
			pDC->LineTo(m_EndPoint);
			
			x1 = x + (int)distance_x;
			y1 = y + (int)distance_y;

			pDC->MoveTo(CPoint(x,y));
			pDC->LineTo(CPoint(x1,y1));
		}
		return;
	}

	// draw the original line
	pDC->MoveTo(m_StartingPoint);
    pDC->LineTo(m_EndPoint);
	
	// define an array of integer to store the drawing point for 4 lines to draw a bridge
	
	distance_porpotion_collection[0] = 3;
	distance_porpotion_collection[1] = distance_porpotion - 2;
	distance_porpotion_collection[2] = distance_porpotion;

	for (int n=0; n<3; n++)
	{
		_Draw(distance_porpotion_collection[n]);
		
		// draw the mirrored line
		pDC->MoveTo(CPoint(x,y));
		pDC->LineTo(CPoint(x1,y1));
	}


	pDC->SelectObject (pOldPen);
}


void CBridgeView::InvertBridge (CDC* pDC, CPoint ptFrom, CPoint ptTo, UINT lines_distance)
{
	int nOldMode = pDC->SetROP2 (R2_NOTXORPEN);
	
	m_StartingPoint = ptFrom;
    m_EndPoint = ptTo;
	distance_porpotion = lines_distance;

	// define the horizontal and vertical distance for the line draw
	distance_x = m_EndPoint.x-m_StartingPoint.x;
	distance_y = m_EndPoint.y-m_StartingPoint.y;

	int distance_porpotion_collection[3];
	distance_porpotion_collection[0] = 2;
	distance_porpotion_collection[1] = distance_porpotion - 2;
	distance_porpotion_collection[2] = distance_porpotion;

	// avoid devide by zero
	if (distance_x == 0)
	{	
		for (int n =0; n<3 ;n++)
		{
			x = m_StartingPoint.x + distance_porpotion_collection[n];
			y = m_StartingPoint.y;

			pDC->MoveTo(m_StartingPoint);
			pDC->LineTo(m_EndPoint);

			x1 = x + (int)distance_x;
			y1 = y + (int)distance_y;
			
			pDC->MoveTo(CPoint(x,y));
			pDC->LineTo(CPoint(x1,y1));
		}
		return;
	}

	// to Disable visual artifact
	if (distance_y == 0)
	{	
		for (int n =0; n<3 ;n++)
		{
			x = m_StartingPoint.x ;
			y = m_StartingPoint.y - distance_porpotion_collection[n];

			pDC->MoveTo(m_StartingPoint);
			pDC->LineTo(m_EndPoint);
			
			x1 = x + (int)distance_x;
			y1 = y + (int)distance_y;

			pDC->MoveTo(CPoint(x,y));
			pDC->LineTo(CPoint(x1,y1));
		}
		return;
	}

	// draw the original line
	pDC->MoveTo(m_StartingPoint);
    pDC->LineTo(m_EndPoint);
	
	// define an array of integer to store the drawing point for 4 lines to draw a bridge
	
	distance_porpotion_collection[0] = 3;
	distance_porpotion_collection[1] = distance_porpotion - 2;
	distance_porpotion_collection[2] = distance_porpotion;

	for (int n=0; n<3; n++)
	{
		_Draw(distance_porpotion_collection[n]);
		
		// draw the mirrored line
		pDC->MoveTo(CPoint(x,y));
		pDC->LineTo(CPoint(x1,y1));
	}


	m_StartingPoint = 0;
    m_EndPoint = 0;    
	
	pDC->SetROP2(nOldMode);
}



void CBridgeView::_Draw (int line_distance)
{

	// Two straight lines is the basic to construct a basic bridge. Since the 
	// mouse only supply two sets of cordinates, which is the starting point
	// and the ending point, to construct a straight line. Another straight
	// line, however, is required to built the bridge. Thus, trigronometri is 
	// is being implement to calculate the mirrored line. It should meet two
	// following requirements : 1, There should be a constant disctance between 
	// two lines  2, Both the starting and ending cordinates of both line should
	// be in 90 degree porpotion to each other.


	double gradiant =  distance_y / distance_x ;

	// To make sure gradiant to always have a possitive value
	if ( gradiant < 0)
		gradiant = gradiant * -1;

	// define the angle for the triangle ,reponsible for transforming the mirrored line
	double alpha = (double) ((pi/2) - (atan(gradiant)));

	// calculate the number of horizontal and vertical pixels to transform the mirrored
	// line into a possition porpotion to the original line
	int horizontal = (int) (line_distance * (sin (alpha)));
	int diagonal = (int) (line_distance * (cos (alpha)));

	// Define the transformation for four types of line direction
	// North-east line 
	if ((distance_x > 0) && (distance_y < 0))
	{
		y = m_StartingPoint.y - horizontal;
		x = m_StartingPoint.x - diagonal;
	}

	// South-west line
	if ((distance_x < 0) && (distance_y > 0))
	{
		y = m_StartingPoint.y + horizontal;
		x = m_StartingPoint.x + diagonal;
	}

	// North-west line
	if ((distance_x < 0) && (distance_y < 0))
	{
		y = m_StartingPoint.y + horizontal;
		x = m_StartingPoint.x - diagonal;
	}

	// South-east line
	if ((distance_x > 0) && (distance_y > 0))
	{
		y = m_StartingPoint.y - horizontal;
		x = m_StartingPoint.x + diagonal;
	}

	// define the other end cordinate for the mirrored line
	x1 = x + (int)distance_x;
	y1 = y + (int)distance_y;

}

void CBridgeView::SelectedBridge (CDC* pDC, CPoint ptFrom, CPoint ptTo)
{
	CPoint ptFrom_upperLeft, ptFrom_lowerRight, ptTo_upperLeft, ptTo_lowerRight;
	
	ptFrom_upperLeft = CPoint(ptFrom.x-3, ptFrom.y-3);
	ptFrom_lowerRight = CPoint(ptFrom.x+3, ptFrom.y+3);
	ptTo_upperLeft = CPoint(ptTo.x-3, ptTo.y-3);
	ptTo_lowerRight = CPoint(ptTo.x+3, ptTo.y+3);

	CPen pen (PS_SOLID, 3, RGB(0,0,0));
    CPen* pOldPen = pDC->SelectObject (&pen);
	pDC->SelectObject (&pen);

	pDC->Rectangle(ptFrom_upperLeft.x, ptFrom_upperLeft.y, ptFrom_lowerRight.x, ptFrom_lowerRight.y);
	pDC->Rectangle(ptTo_upperLeft.x, ptTo_upperLeft.y, ptTo_lowerRight.x, ptTo_lowerRight.y);

	pDC->SelectObject (pOldPen);
}