// CLine.cpp: implementation of the CLine class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Map Studio MAX R1.h"
#include "CLine.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL (CLine, CObject, 1)

CLine::CLine (CPoint ptFrom, CPoint ptTo, UINT nWidth, COLORREF crColor)
{
    m_StartingPoint = ptFrom;
    m_EndPoint = ptTo;
    m_LineWidth = nWidth;
    m_LineColor = crColor;
}

void CLine::Serialize (CArchive& ar)
{
    CObject::Serialize (ar);

    if (ar.IsStoring ())
        ar << m_StartingPoint << m_EndPoint << m_LineWidth << (DWORD) m_LineColor;
    else
        ar >> m_StartingPoint >> m_EndPoint >> m_LineWidth >> (DWORD) m_LineColor;
}

// Get methods
CPoint CLine::GetStartingPoint()
{
	return m_StartingPoint;
}

CPoint CLine::GetEndPoint()
{
	return m_EndPoint;
}

UINT CLine::GetLineWidth()
{
	return m_LineWidth;
}

COLORREF CLine::GetLineColor()
{
	return m_LineColor;
}

// Set methods
void CLine::SetStartingPoint(CPoint start_point)
{
	m_StartingPoint = start_point;
}

void CLine::SetEndPoint(CPoint end_point)
{
	m_EndPoint = end_point;
}

void CLine::SetLineWidth(UINT line_width)
{
	m_LineWidth = line_width;
}

void CLine::SetLineColor(COLORREF line_color)
{
	m_LineColor = line_color;
}

bool CLine::HitTest(CPoint point)
{
    // check handles
    CRect R;

    R = CRect(m_StartingPoint, m_EndPoint );
    R.NormalizeRect();
    CRect VicX = R;
    CRect VicY = R;

    VicX.InflateRect( (m_LineWidth + 5), 0 );
    VicY.InflateRect( 0, (m_LineWidth + 5) );

    if (!VicX.PtInRect(point) && !VicY.PtInRect(point))
		return false;

    // check distance from the point to the line by x and by y
    
	int dx = m_EndPoint.x - m_StartingPoint.x;
    int dy = m_EndPoint.y - m_StartingPoint.y;
    double r = sqrt(dx*dx + dy*dy);

    // compute coordinates relative to the origin of the line
    int x1 = point.x - m_StartingPoint.x;
    int y1 = point.y - m_StartingPoint.y;

    // compute coordinates relative to the line
    double x2 = (x1*dx + y1*dy)/r;
    double y2 = (-x1*dy + y1*dx)/r;

    if ((x2>=0) && (x2<=r) && (y2<=(m_LineWidth + 5)) && (y2 >= -((int)m_LineWidth + 5)))
        return true;

    return false;
}

