// Rail.cpp: implementation of the CRail class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Map Studio MAX R1.h"
#include "Rail.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL (CRail, CObject, 1)

CRail::CRail (CPoint ptFrom, CPoint ptTo, UINT nWidth, COLORREF crColor)
{
    m_StartingPoint = ptFrom;
    m_EndPoint = ptTo;
    m_RailWidth = nWidth;
    m_RailColor = crColor;
}

void CRail::Serialize (CArchive& ar)
{
    CObject::Serialize (ar);

    if (ar.IsStoring ())
        ar << m_StartingPoint << m_EndPoint << m_RailWidth << (DWORD) m_RailColor;
    else
        ar >> m_StartingPoint >> m_EndPoint >> m_RailWidth >> (DWORD) m_RailColor;
}

// Get methods
CPoint CRail::GetStartingPoint()
{
	return m_StartingPoint;
}

CPoint CRail::GetEndPoint()
{
	return m_EndPoint;
}

UINT CRail::GetRailWidth()
{
	return m_RailWidth;
}

COLORREF CRail::GetRailColor()
{
	return m_RailColor;
}

// Set methods
void CRail::SetStartingPoint(CPoint start_point)
{
	m_StartingPoint = start_point;
}

void CRail::SetEndPoint(CPoint end_point)
{
	m_EndPoint = end_point;
}

void CRail::SetRailWidth(UINT line_width)
{
	m_RailWidth = line_width;
}

void CRail::SetRailColor(COLORREF line_color)
{
	m_RailColor = line_color;
}

bool CRail::HitTest(CPoint point)
{
    // check handles
    CRect R;

    R = CRect(m_StartingPoint, m_EndPoint );
    R.NormalizeRect();
    CRect VicX = R;
    CRect VicY = R;

    VicX.InflateRect( (m_RailWidth + 5), 0 );
    VicY.InflateRect( 0, (m_RailWidth + 5) );

    if (!VicX.PtInRect(point) && !VicY.PtInRect(point))
		return false;

    // check distance from the point to the line by x and by y
    
	int dx = m_EndPoint.x - m_StartingPoint.x;
    int dy = m_EndPoint.y - m_StartingPoint.y;
    double r = sqrt(dx*dx + dy*dy);

    // compute coordinates relative to the origin of the line
    int x1 = point.x - m_StartingPoint.x;
    int y1 = point.y - m_StartingPoint.y;

    // compute coordinates relative to the line
    double x2 = (x1*dx + y1*dy)/r;
    double y2 = (-x1*dy + y1*dx)/r;

    if ((x2>=0) && (x2<=r) && (y2<=(m_RailWidth + 5)) && (y2 >= -((int)m_RailWidth + 5)))
        return true;

    return false;
}

